#include <math.h>
#include <string.h>
#include "cpcsolver.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "globmat.h"
#include "globmatt.h"
#include "globmatc.h"
#include "elemswitcht.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "loadcase.h"
#include "mechprint.h"
#include "transprint.h"
#include "mtsolver.h"


/**
   function solves partially coupled thermo-hydro-mechanical time-dependent problem
   for growing constructions
*/
void solve_gpcouplprob ()
{

  switch (Cp->tnlinsol){
  case newtonc:{
    newton_raphson_gparcoupl (0);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown solver of nonlinear equation system is required");
    fprintf (stderr,"\n in function nonlinear_solver (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function solves partially coupled thermo-hydro-mechanical time-dependent problem
   for growing constructions
*/
void newton_raphson_gparcoupl (long lcid)
{
  switch (Tp->tprob){
  case growing_np_problem:{
    newton_raphson_gparcoupl_lin (lcid);
    break;
  }
  case growing_np_problem_nonlin:{
    newton_raphson_gparcoupl_nonlin (lcid);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown METR problem type is required in function newton_raphson_parcoupl (file %s, line %d).\n",
	     __FILE__,__LINE__);
  }
  }
}

/**
   Function solves system of linear TRFEL algebraic and non-linear MEFEL equations by Newton-Raphson method
   for time-dependent problems
   
   @param lcid - load case id
   
   14.5.2006 JK, corrected by TKr 5.4.2007
*/
void newton_raphson_gparcoupl_lin (long lcid)
{
  long i,ii,j,k,nm,nt,mncd,tncd,mnce,tnce,ini;
  double zero,dt,prev_timem,prev_timet,end_time,alpha,s;
  double *dr,*fb,*r,*d,*f,*fp,*fi,*p,*lhst,*tdlhst,*rhst;
  double norf, norfa, err;  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);

  aux = fopen (fname,"w");

  //  maximum number of iterations in inner loop
  ini = Cp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Cp->errnr;
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;

  /* *************************** */
  /*  vectors of transport part  */
  /* *************************** */
  //  left hand side
  lhst = Lsrst->give_lhs (0);
  tdlhst = Lsrst->give_tdlhs (0);
  //  right hand side
  rhst = Lsrst->give_rhs (0);
  
  //  array containing predictors
  d = new double [nt];
  //  auxiliary vector 
  p = new double [nt];
  
  //  initial values
  nullv (lhst,nt);
  nullv (tdlhst,nt);
  nullv (d,nt);
  nullv (p,nt);

  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */
  fp = new double [nm];
  fb = new double [nm];
  fi = new double [nm];
  dr = new double [nm];

  //  initial values
  nullv (fp,nm);
  nullv (fb,nm);
  nullv (fi,nm);
  nullv (dr,nm);

  //  initialization phase
  r = Lsrs->give_lhs (lcid);
  f = Lsrs->give_rhs (lcid);
    
  alpha=Tp->alpha;
  zero=Tp->zero;
  
  //driving time from METR
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
    
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  //  konec provizoria??!!

  Mp->time = Cp->time;
  prev_timem = Mp->time;
  Mp->timecon.take_values (Mp->timecon);
  Tp->time = Cp->time;
  prev_timet = Tp->time;
  Tp->timecont.take_values (Cp->timecon);

  //  nodes - integration points interpolation
  approximation ();  
  approximation_temper ();
  approximation_humid ();

  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  //  initiation of transport material models
  Tm->initmaterialmodels();

  Gtm->nm = new long [Gtm->nn];
  
  // **************************
  //  main iteration loop  ***
  // **************************
  i=0;
  ii=0;
  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();  

  //saving values for the first iteration
  Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);
  Mt->lhs_save (r,1);

  // *********************************************************
  //                     transport part
  // *********************************************************
  do{
    
    if (Mesprc==1)  fprintf (stdout,"\n\n ------------------------------------------------------------------------------");
    fprintf (stdout,"\n metr time %e,    trfel time %e,   mefel time %e",Cp->time,Tp->time,Mp->time);
    fprintf (stdout,"\n time increment %lf",dt);
    if (Mesprc==1)  fprintf (stdout,"\n -----------------------------------------------------------------------------\n");

    if (Mesprc==1)  fprintf (stdout,"\n\n provadi se TRFEL,  trfel time %e",Tp->time);
    
    //  searching for new elements
    //  only new elements are switched on
    //  former elements are temporarily switched off
    tnce = Gtt->search_newelem (Tp->time,prev_timet);
    
    //  searching for new DOFs
    //  only new DOFs are switched on
    //  former DOFs are temporarily switched off
    tncd = Gtt->search_newdofs (Tp->time,prev_timet);
 
    //  marking of elements switched on
    Gtt->update_elem (Tp->time);
    
    //  marking of nodes switched on
    Gtt->update_nodes ();
    
    //  marking of DOFs switched on
    Gtt->update_dofs (Tp->time);
    
    //  generation of new code numbers
    Ndoft = Gtt->codenum_generation (Outt);
    nt=Ndoft;
            
    Tm->updateipval ();
    i++;

    if (tnce!=0 || tncd!=0){
      //  assembling of displacement vector for new code numbers
      Tt->lhs_restore (lhst,Lsrst->lhsi,tdlhst);
    }

    //  cleaning of the conductivity matrix
    if (Kmat != NULL){
      delete Kmat;
      Kmat=NULL;
    }
    //  cleaning of the capacity matrix
    if (Cmat != NULL){
      delete Cmat;
      Cmat=NULL;
    }

 
    //  assignment of initial nodal values
    //  it works for problem with growing number of elements
    //  it does not work generally for problems with increasing and decreasing number of elements
    if (tnce>0){
      Tt->initial_nodval ();
    }

    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    for (j=0;j<nt;j++){
      p[j] = lhst[j] + (1.0-alpha)*dt*tdlhst[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhst,nt);
    
    //  auxiliary vector
    Kmat->gmxv (p,d);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    for (j=0;j<nt;j++){
      rhst[j] = rhst[j] - d[j];
      d[j]=tdlhst[j];
    }
    
    //  solution of the system of algebraic equations
    Tp->ssle->solve_system (Gtt,Kmat,tdlhst,rhst,Outt);

    for (j=0;j<nt;j++){
      s=(1.0-alpha)*d[j]+alpha*tdlhst[j];
      lhst[j]+=dt*s;
    }
    
    //  physically corrected solution
    solution_correction ();    

    Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);

    //  approximation of nodal values into ontegration points
    approximation ();
    
    if (Mesprc==1){
      //cleaning once again for more free memmory
      //  cleaning of the conductivity matrix
      if (Kmat != NULL){
	delete Kmat;
	Kmat=NULL;
      }
      //  cleaning of the capacity matrix
      if (Cmat != NULL){
	delete Cmat;
	Cmat=NULL;
      }
    }

    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************
    if (Mp->time <= Tp->time){
      
      //if (Mesprc==1)  
	fprintf (stdout,"\n\n provadi se MEFEL,  mefel time %e\n",Mp->time);
            
      //  searching for new elements
      //  only new elements are switched on
      //  former elements are temporarily switched off
      mnce = Gtm->search_newelem (Mp->time,prev_timem);
    
      if (mnce!=0){
	//  attained displacements have to be saved on new elements
	//  otherwise stresses will not be correct
	Mt->initial_displ ();
      }
      
      //  searching for new DOFs
      //  only new DOFs are switched on
      //  former DOFs are temporarily switched off
      mncd = Gtm->search_newdofs (Mp->time,prev_timem);
      
      //  marking of elements switched on
      Gtm->update_elem (Mp->time);
      
      //  marking of nodes switched on
      Gtm->update_nodes ();
      
      //  marking of DOFs switched on
      Gtm->update_dofs (Mp->time);
      
      //  definition of DOFs
      Ndofm = Gtm->codenum_generation (Out);
      nm=Ndofm;
      
      //  update of nodes and DOFs
      //if (mncd>0){
      if (Smat != NULL){
	delete Smat;
	Smat=NULL;
      }
      //}
      
      //  assembling of lhs array for new code numbers
      //if (mnce!=0 || mncd!=0){
      Mt->lhs_restore (r,nm);
      Mt->rhs_restore (fp,nm);
      //}
      
      
      approximation_temper ();
      approximation_humid ();

      Mm->updateipval();

      //fprintf (stdout,"\nright hand side\n\n");
      //  assembling of the right hand side (prescribed loading)
      mefel_right_hand_side (lcid,f);
      
      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      
      Mp->phase=1;
      //  computation of unbalanced forces which will be on the right hand side
      internal_forces (lcid,fi);
      
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
	fp[j]=f[j];
      }    
      
      //  assembling of tangent stiffness matrix
      //fprintf (stdout,"\nphase1 = stiffness_matrix\n\n");
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      Mp->phase=2;

      //  update of computed values
      internal_forces (lcid,fi);

      norfa=ss(f,f,nm);
      for (j=0;j<nm;j++){
	fb[j] = f[j] - fi[j];
      }
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,nm);
      if (norfa > 0.0)
	norf /= norfa;
      
      if (norf<err){

	//  backup of nodal values because code numbers may be changed
	Mt->lhs_save (r,1);
	Mt->rhs_save (f);

	// computation of values required for output    
	compute_req_val (lcid);
	//  time increment
	prev_timem=Mp->time;
	Mp->time = Mp->timecon.newtime ();
	Mp->timecon.take_values (Mp->timecon);

	if (Mp->timecon.isitimptime ()==1){
	  mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	}
	
	print_step(lcid, ii, Mp->time, f);
	print_flush();
	ii++;	
      }
      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      else{
	for (j=0;j<ini;j++)
	  {
	    
	    if(Cp->fcsolv == fullnewtonc){
	      //cleaning of stiffness matrix
	      if (Smat != NULL){
		delete Smat;
		Smat=NULL;
	      }
	      //assembling of tangent stiffness matrix for fullnewton
	      stiffness_matrix (lcid);
	    }
	    
	    //  solution of K(r).v=F
	    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

	    for (k=0;k<nm;k++)
	      r[k]+=dr[k];
	    
	    //  computation of internal forces
	    internal_forces (lcid,fi);
	    
	    //  vector of unbalanced forces
	    for (k=0;k<nm;k++)
	      fb[k]=f[k]-fi[k];
	    
	    //  norm of vector of unbalanced forces
	    norf=ss(fb,fb,nm);
	    if (norfa > 0.0)
	      norf /= norfa;
	    
	    //if (Mesprc==1) 
	      fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e\n",Mp->time,j,norf);
	    if (norf<err)  break;
	  }
	if (j==ini || norf>err)
	  {
	    //if (Mesprc==1) 
	      fprintf (stdout,"\n Iteration process does not converge req. err = %e, reached err = %e\n", err,norf);

	    //backup of norf in case of unbalanced internal forces
	    err = norf;

	    //  backup of nodal values because code numbers may be changed
	    Mt->lhs_save (r,1);
	    Mt->rhs_save (f);
	    
	    // equilibrium state was reached not
	    compute_req_val (lcid);
	    //  time increment
	    prev_timem = Mp->time;
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
	else
	  {
	    //  backup of nodal values because code numbers may be changed
	    Mt->lhs_save (r,1);
	    Mt->rhs_save (f);
	    
	    // equilibrium state was reached
	    compute_req_val (lcid);
	    //  time increment
	    prev_timem = Mp->time;
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);

	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (r,fp,ii,Mp->time,dt,nm);

	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
      }
      
      if (Mesprc==1){
	//cleaning once again for more free memmory
	if (Smat != NULL){
	  delete Smat;
	  Smat=NULL;
	}
      }
    }

    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    prev_timet = Tp->time;
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);
    dt = Cp->timecon.actualbacktimeincr ();
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();

  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] dr;

  delete [] p;
  delete [] d;
}


/**
   Function solves system of non-linear TRFEL algebraic and non-linear MEFEL equations by Newton-Raphson method
   for time-dependent problems.
   
   @param lcid - load case id
   
   TKr 5.4.2007
*/
void newton_raphson_gparcoupl_nonlin (long lcid)
{
  long i,ii,j,k,nm,nt,mncd,tncd,mnce,tnce,ini;
  double zero,dt,prev_timem,prev_timet,end_time,alpha;
  double *dr,*fb,*r,*d,*f,*fp,*fi,*p,*lhst,*tdlhst,*rhst;
  double *fbt,*fit,*lhst_last;
  //double s;
  double norf_last;
  double norf, norfa, err;  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  FILE *aux;

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf (fname,"%s%s.bac",path, name);

  aux = fopen (fname,"w");

  //  maximum number of iterations in inner loop
  ini = Cp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Cp->errnr;
  //  number of mechanical degrees of freedom
  nm=Ndofm;
  //  number of transport degrees of freedom
  nt=Ndoft;

  /* *************************** */
  /*  vectors of transport part  */
  /* *************************** */
  //  left hand side
  lhst = Lsrst->give_lhs (0);
  tdlhst = Lsrst->give_tdlhs (0);
  //  right hand side
  rhst = Lsrst->give_rhs (0);
  
  //  array containing predictors
  d = new double [nt];
  //  auxiliary vector 
  p = new double [nt];
  fbt = new double [nt];
  fit = new double [nt];
  lhst_last = new double [nt];

  //  initial values
  nullv (lhst,nt);
  nullv (tdlhst,nt);
  nullv (d,nt);
  nullv (p,nt);
  nullv (fbt,nt);
  nullv (fit,nt);

  /* **************************** */
  /*  vectors of mechanical part  */
  /* **************************** */
  fp = new double [nm];
  fb = new double [nm];
  fi = new double [nm];
  dr = new double [nm];
  
  //  initial values
  nullv (fp,nm);
  nullv (fb,nm);
  nullv (fi,nm);
  nullv (dr,nm);

  //  initialization phase
  r = Lsrs->give_lhs (lcid);
  f = Lsrs->give_rhs (lcid);
    
  alpha=Tp->alpha;
  zero=Tp->zero;
  
  //driving time from METR
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
    
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  //  konec provizoria??!!

  Mp->time = Cp->time;
  prev_timem = Mp->time;
  Mp->timecon.take_values (Mp->timecon);
  Tp->time = Cp->time;
  prev_timet = Tp->time;
  Tp->timecont.take_values (Cp->timecon);

  //  nodes - integration points interpolation
  approximation ();  
  approximation_temper ();
  approximation_humid ();

  //  initiation of mechanical material models
  Mm->initmaterialmodels();
  //  initiation of transport material models
  Tm->initmaterialmodels();

  Gtm->nm = new long [Gtm->nn];

  // **************************
  //  main iteration loop  ***
  // **************************
  i=0;
  ii=0;
  print_init(-1, "wt");
  print_step(lcid,ii,Mp->time,f);
  print_flush();
  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);
  print_flusht();  

  //saving values for the first iteration
  Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);
  Mt->lhs_save (r,1);

  // *********************************************************
  //                     transport part
  // *********************************************************
  do{
    
    if (Mesprc==1)  fprintf (stdout,"\n\n ------------------------------------------------------------------------------");
    fprintf (stdout,"\n metr time %e,    trfel time %e,   mefel time %e",Cp->time,Tp->time,Mp->time);
    fprintf (stdout,"\n time increment %lf",dt);
    if (Mesprc==1)  fprintf (stdout,"\n -----------------------------------------------------------------------------\n");

    if (Mesprc==1)  fprintf (stdout,"\n\n provadi se TRFEL,  trfel time %e",Tp->time);
    
    //  searching for new elements
    //  only new elements are switched on
    //  former elements are temporarily switched off
    tnce = Gtt->search_newelem (Tp->time,prev_timet);
    
    //  searching for new DOFs
    //  only new DOFs are switched on
    //  former DOFs are temporarily switched off
    tncd = Gtt->search_newdofs (Tp->time,prev_timet);
 
    //  marking of elements switched on
    Gtt->update_elem (Tp->time);
    
    //  marking of nodes switched on
    Gtt->update_nodes ();
    
    //  marking of DOFs switched on
    Gtt->update_dofs (Tp->time);
    
    //  generation of new code numbers
    Ndoft = Gtt->codenum_generation (Outt);
    nt=Ndoft;
            
    Tm->updateipval ();
    i++;

    if (tnce!=0 || tncd!=0){
      //  assembling of displacement vector for new code numbers
      Tt->lhs_restore (lhst,Lsrst->lhsi,tdlhst);
    }

    //  cleaning of the conductivity matrix
    if (Kmat != NULL){
      delete Kmat;
      Kmat=NULL;
    }
    //  cleaning of the capacity matrix
    if (Cmat != NULL){
      delete Cmat;
      Cmat=NULL;
    }

 
    //  assignment of initial nodal values
    //  it works for problem with growing number of elements
    //  it does not work generally for problems with increasing and decreasing number of elements
    if (tnce>0){
      Tt->initial_nodval ();
    }

    //  conductivity matrix
    conductivity_matrix (lcid);
    
    //  capacity matrix
    capacity_matrix (lcid);
    
    for (j=0;j<nt;j++){
      p[j] = lhst[j] + (1.0-alpha)*dt*tdlhst[j];
    }
    
    //  right hand side of transport part
    trfel_right_hand_side (0,rhst,nt);
    
    //  auxiliary vector
    Kmat->gmxv (p,d);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    for (j=0;j<nt;j++){
      rhst[j] = rhst[j] - d[j];
    }
    
    //  solution of the system of algebraic equations
    Tp->ssle->solve_system (Gtt,Kmat,tdlhst,rhst,Outt);

    for (j=0;j<nt;j++){
      //verze1:
      //s=(1.0-alpha)*d[j]+alpha*tdlhst[j];
      //lhst[j]+=dt*s;
      //verze2:
      lhst[j] = p[j] + alpha*dt*tdlhst[j];
    }
    
    //  physically corrected solution
    solution_correction ();    

    Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);

    //  approximation of nodal values into ontegration points
    approximation ();
    
    norf_last = 1.0e20;    
    //inner iteration loop//////////////////
    for (j=0;j<ini;j++){
      
      //  physically corrected solution
      solution_correction ();
      
      Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);
      
      //  approximation of nodal values into ontegration points
      approximation ();
      
      // full newton-raphson
      if (Tp->trsolv == fullnewtont){
	// matrices are computing in each inner iteration
	//  cleaning of the conductivity matrix
	if (Kmat != NULL){
	  delete Kmat;
	  Kmat=NULL;
	}
	//  cleaning of the capacity matrix
	if (Cmat != NULL){
	  delete Cmat;
	  Cmat=NULL;
	}
	//  capacity matrix
	capacity_matrix (0);
	
	//  conductivity matrix
	conductivity_matrix (0);
	
	//  auxiliary vector  K (d+(1-alpha)*dt*v)
	Kmat->gmxv (p,d);
	
	//  matrix of the system of equations
	//  C + alpha.dt.K
	Kmat->scalgm (dt*alpha);
	Kmat->addgm (1.0,*Cmat);
      }
      
      if (Tp->trestype==lrhst){
	//  right hand side of transport part
	trfel_right_hand_side (0,rhst,nt);
	// Solver computes residuum from system of equations
	Kmat->gmxv (tdlhst,fit);
	// Res. = F - K (d+(1-alpha)*dt*v) - (C + alpha*dt*K)*v 
	for (k=0;k<nt;k++){
	  fbt[k] = rhst[k] - d[k] - fit[k];
	}
      }
      
      if (Tp->trestype==fluxest){
	// Solver computes unbalanced fluxes
	internal_fluxes (fit,nt);
	//  vector of unbalanced fluxes
	for (k=0;k<nt;k++){
	  fbt[k]=fit[k];
	}
      }
      
      norf = ss (fbt,fbt,nt);
      
      if (Mesprc==1)  fprintf (stdout,"\n inner iteration %ld   error %e",j,norf);
      
      if (norf<err) break;
      
      Tp->ssle->solve_system (Gtt,Kmat,d,fbt,Outt);
      
      for (k=0;k<nt;k++){
	tdlhst[k]+=d[k];
	lhst[k]+=alpha*dt*d[k];
      }
      
      //  physically corrected solution
      solution_correction ();
      
      Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);
      
      //  approximation of nodal values into ontegration points
      approximation ();
      
      //  convergence control for newton-raphson
      //  condition of decreesing of error
      if (Tp->convergcontrolt==yes){
	if (norf > norf_last){
	  for (k=0;k<nt;k++){
	    lhst[k] = lhst_last[k]; //storing results from previous inner step
	  }
	  
	  //  physically corrected solution
	  solution_correction ();
	  
	  Tt->lhs_save (lhst,Lsrst->lhsi,tdlhst);
	  
	  //  approximation of nodal values into ontegration points
	  approximation ();	  

	  if (Mesprc==1)  fprintf (stdout,"\n\n convergence control: inner iteration skiped %ld error %e\n\n",j,norf);	  
	  
	  break;
	}
	for (k=0;k<nt;k++){
	  lhst_last[k] = lhst[k]; //storing results from previous inner step
	}
	
	norf_last = norf; //storing norf from previous inner step
      }
    }
    ////////////////////////////////////////
    
    if (Mesprc==1){
      //cleaning once again for more free memmory
      //  cleaning of the conductivity matrix
      if (Kmat != NULL){
	delete Kmat;
	Kmat=NULL;
      }
      //  cleaning of the capacity matrix
      if (Cmat != NULL){
	delete Cmat;
	Cmat=NULL;
      }
    }

    // *********************************************************
    //  end of transport part and beginning of mechanical part
    // *********************************************************
    if (Mp->time <= Tp->time){
      
      if (Mesprc==1)  fprintf (stdout,"\n\n provadi se MEFEL,  mefel time %e",Mp->time);
            
      //  searching for new elements
      //  only new elements are switched on
      //  former elements are temporarily switched off
      mnce = Gtm->search_newelem (Mp->time,prev_timem);
    
      if (mnce!=0){
	//  attained displacements have to be saved on new elements
	//  otherwise stresses will not be correct
	Mt->initial_displ ();
      }
      
      //  searching for new DOFs
      //  only new DOFs are switched on
      //  former DOFs are temporarily switched off
      mncd = Gtm->search_newdofs (Mp->time,prev_timem);
      
      //  marking of elements switched on
      Gtm->update_elem (Mp->time);
      
      //  marking of nodes switched on
      Gtm->update_nodes ();
      
      //  marking of DOFs switched on
      Gtm->update_dofs (Mp->time);
      
      //  definition of DOFs
      Ndofm = Gtm->codenum_generation (Out);
      nm=Ndofm;
      
      //  update of nodes and DOFs
      //if (mncd>0){
      if (Smat != NULL){
	delete Smat;
	Smat=NULL;
      }
      //}
      
      //  assembling of lhs array for new code numbers
      //if (mnce!=0 || mncd!=0){
      Mt->lhs_restore (r,nm);
      Mt->rhs_restore (fp,nm);
      //}
      
      
      approximation_temper ();
      approximation_humid ();

      Mm->updateipval();

      //fprintf (stdout,"\nright hand side\n\n");
      //  assembling of the right hand side (prescribed loading)
      mefel_right_hand_side (lcid,f);
      
      //  computation of sums of force components in particular directions
      //  it is used in special creep and consolidation models
      Mb->comp_sum (f);
      
      Mp->phase=1;
      //  computation of unbalanced forces which will be on the right hand side
      internal_forces (lcid,fi);
      
      for (j=0;j<nm;j++){
	fb[j]=f[j]-fp[j]+fi[j];
	fp[j]=f[j];
      }    
      
      //  assembling of tangent stiffness matrix
      //fprintf (stdout,"\nphase1 = stiffness_matrix\n\n");
      stiffness_matrix (lcid);
      
      //  solution of K(r).v=F
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      //  new displacement vector
      for (j=0;j<nm;j++){
	r[j]+=dr[j];
      }
      
      Mp->phase=2;

      //  update of computed values
      internal_forces (lcid,fi);

      norfa=ss(f,f,nm);
      for (j=0;j<nm;j++){
	fb[j] = f[j] - fi[j];
      }
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,nm);
      if (norfa > 0.0)
	norf /= norfa;
      
      if (norf<err){

	//  backup of nodal values because code numbers may be changed
	Mt->lhs_save (r,1);
	Mt->rhs_save (f);

	// computation of values required for output    
	compute_req_val (lcid);
	//  time increment
	prev_timem=Mp->time;
	Mp->time = Mp->timecon.newtime ();
	Mp->timecon.take_values (Mp->timecon);

	if (Mp->timecon.isitimptime ()==1){
	  mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	}
	
	print_step(lcid, ii, Mp->time, f);
	print_flush();
	ii++;	
      }
      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      else{
	for (j=0;j<ini;j++)
	  {

	    if(Cp->fcsolv == fullnewtonc){
	      //cleaning of stiffness matrix
	      if (Smat != NULL){
		delete Smat;
		Smat=NULL;
	      }
	      //assembling of tangent stiffness matrix for fullnewton
	      stiffness_matrix (lcid);
	    }

	    //  solution of K(r).v=F
	    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
	    
	    for (k=0;k<nm;k++)
	      r[k]+=dr[k];
	    
	    //  computation of internal forces
	    internal_forces (lcid,fi);
	    
	    //  vector of unbalanced forces
	    for (k=0;k<nm;k++)
	      fb[k]=f[k]-fi[k];
	    
	    //  norm of vector of unbalanced forces
	    norf=ss(fb,fb,nm);
	    if (norfa > 0.0)
	      norf /= norfa;
	    
	    if (Mesprc==1) fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
	    if (norf<err)  break;
	  }
	if (j==ini || norf>err)
	  {
	    if (Mesprc==1) fprintf (stdout,"\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);

	    //backup of norf in case of unbalanced internal forces
	    err = norf;

	    //  backup of nodal values because code numbers may be changed
	    Mt->lhs_save (r,1);
	    Mt->rhs_save (f);
	    
	    // equilibrium state was reached not
	    compute_req_val (lcid);
	    //  time increment
	    prev_timem = Mp->time;
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);
	    
	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (r,fp,ii,Mp->time,dt,nm);
	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
	else
	  {
	    //  backup of nodal values because code numbers may be changed
	    Mt->lhs_save (r,1);
	    Mt->rhs_save (f);
	    
	    // equilibrium state was reached
	    compute_req_val (lcid);
	    //  time increment
	    prev_timem = Mp->time;
	    Mp->time = Mp->timecon.newtime ();
	    Mp->timecon.take_values (Mp->timecon);

	    if (Mp->timecon.isitimptime ()==1)
	      mtsolver_save (r,fp,ii,Mp->time,dt,nm);

	    print_step(lcid, ii, Mp->time, f);
	    print_flush();
	    ii++;
	  }
      }
      
      if (Mesprc==1){
	//cleaning once again for more free memmory
	if (Smat != NULL){
	  delete Smat;
	  Smat=NULL;
	}
      }
    }

    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    prev_timet = Tp->time;
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);
    dt = Cp->timecon.actualbacktimeincr ();
    
    print_stept(0,i,Tp->time,NULL);
    print_flusht();

  }while(Cp->time<=end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fi;
  delete [] fb;
  delete [] fp;
  delete [] dr;

  delete [] p;
  delete [] d;

  delete [] fit;
  delete [] fbt;
  delete [] lhst_last;
}
