#include <stdio.h>
#include <stdlib.h>

#include "metrinit.h"

#include "globalc.h"
#include "globmatc.h"
#include "fcsolver.h"
#include "gtopology.h"

#include "seqfilesm.h"
#include "seqfilest.h"

#include "iotools.h"
#include "xfile.h"
#include "outdriverc.h"


void metr_init (char *argv)
{
  long i,merr;
  XFILE *in,*inm,*intr;

  fprintf (stderr,"\n\n ******************************************************\n");
  
  //  docasne umisteni
  if (Bar2d==NULL)   Bar2d = new barel2d;
  if (Beam2d==NULL)   Bar2d = new barel2d;
  if (Pelq==NULL)    Pelq = new planeelemlq;
  if (Peqq==NULL)    Peqq = new planeelemqq;
  if (Asymlq==NULL)  Asymlq = new axisymlq;
  if (Lhex==NULL)  Lhex = new linhex;
  
  if (Ltet==NULL)  Ltet = new lintet;
  if (Bar3d==NULL)   Bar3d = new barel3d;

  //  problem description of all coupled problem
  Cp = new probdescc;
  //  problem description of mechanical part of problem
  Mp = new probdesc;
  //  problem description of thermo-hydro part of problem
  Tp = new probdesct;
  //  topology of mechanical part of problem
  Gtm = new gtopology;
  //  topology of thermo-hydro part of problem
  Gtt = new gtopology;
  //  auxiliary united topology (mechanical + thermo-hydro)
  Gtu = new gtopology;
  //  additional topological data about mechanical part of problem
  Mt = new mechtop;
  //  additional topological data about thermo-hydro part of problem
  Tt = new transtop;
  //  additional topological data about thermo-hydro part of problem
  Ct = new couptop;
  //  materials of mechanical part of problem
  Mm = new mechmat;
  //  materials of thermo-hydro part of problem
  Tm = new transmat;
  //  materials of thermo-hydro part of problem
  Cmu = new coupmatu;
  //  materials of thermo-hydro part of problem
  Cml = new coupmatl;
  //  cross-section in mechanical part of problem
  Mc = new mechcrsec;
  //  cross-section in thermo-hydro part of problem
  Tc = new transcrsec;
  //  boundary conditions and load cases in coupled problem
  Cb = new coupbclc;
  //  boundary conditions and load cases in mechanical part of problem
  Mb = new mechbclc;
  //  boundary conditions and load cases in thermo-hydro part of problem
  Tb = new transbclc;
  //outdriver
  Outdm = new outdriverm;
  Outdt = new outdrivert;
  Outdc = new outdriverc;
 
  
  in = xfopen(argv,"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    abort ();
  }
  in->maxlnsize = 1000;
  in->warrning = 1;
  in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;

  //  problem description of all coupled problem
  Cp->read (in);
  
  //  input file for mechanical part
  inm = xfopen(Cp->minfile,"r");
  if (inm==NULL){
    fprintf (stderr,"\n Input file for mechanical part has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    abort ();
  }
  inm->maxlnsize = 1000;
  inm->warrning = 1;
  //intr->kwdmode = sequent_mode;
  inm->kwdmode = ignore;
  inm->ignorecase = 1;

  //  input file for thermo-hydro part
  intr = xfopen(Cp->tinfile,"r");
  if (intr==NULL){
    fprintf (stderr,"\n Input file for thermo-hydro part has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    abort ();
  }
  intr->maxlnsize = 1000;
  intr->warrning = 1;
  //intr->kwdmode = sequent_mode;
  intr->kwdmode = ignore;
  intr->ignorecase = 1;
  
  //  basic problem data
  Mp->read (inm);
  Tp->read (intr);

    
  if ((Cp->tprob==growing_par_coupl_mech_trans) || (Cp->tprob==par_coupl_mech_trans)){
    //  initial time setting
    Cp->time=Cp->timecon.starttime ();
    Tp->time=Tp->timecont.starttime ();
    Mp->time=Mp->timecon.starttime ();
  }

  
  Outc = fopen ("metr.log","wt");
  Out  = fopen ("mefel.log","wt");
  Outt = fopen ("trfel.log","wt");
  
  //  input of node and element data
  Mt->read (inm);
  Tt->read (intr);
  Ct->read (in);
  
    
  // mefel mesh control TKr
  if (Mp->tprob == growing_mech_structure){
    if (Mespr==1){
      merr=Mt->mesh_check();
      if (merr > 0){
	fprintf (stdout,"MEFEL Mesh error = %ld\n\n\n",merr);
	exit(0);
      }
    }
  }

    
  // trfel mesh control
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    if (Mesprt==1){
      merr=Tt->mesh_check();
      if (merr > 0){
	fprintf (stdout,"TRFEL Mesh error = %ld\n\n\n",merr);
	exit(0);
      }
    }
  }


  // metr mesh control
  /*
  if ((Cp->tprob==growing_par_coupl_mech_trans) || (Cp->tprob==par_coupl_mech_trans)){
    if (Mesprc==1){
      merr=Ct->mesh_check();
      if (merr > 0){
	fprintf (stdout,"METR Mesh error = %ld\n\n\n",merr);
	exit(0);
      }
    }
  }
  */

  if (Cp->tprob==fully_coupled_mech_trans){
    //  unification of two topologies
    Gtu->comptop (Gtm,Gtt);
  }
  
  //  generation of code numbers in mechanical part
  Ndofm = Gtm->codenum_generation (Out);//Ndofm=Gtm->gencodnum ();
  //Mt->store_code_num_elem(); // enable combination of plane elements, springs and bar elems with beams//tady????!!

  //  generation of code numbers in transport part
  Ndoft = Gtt->codenum_generation (Outt);//Ndoft=Gtt->gencodnum ();
  
  if (Cp->tprob==fully_coupled_mech_trans){
    //  generation of code numbers in both parts
    Ndofc = Gtu->codenum_generation (Outc);//Ndofc=Gtu->gencodnum ();
    //  distribution of code numbers to local topologies
    Gtu->cndistr (Gtm,Gtt);
    
    Ndofm=Ndofc;
    Ndoft=Ndofc;
    
    if (Mesprc==1)
      fprintf (stdout,"\n total number of degrees of freedom    %ld",Ndofc);
  }

  //read trfel function
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    Gtt->read_gf (intr);
  }

  //  input of material characteristics
  Mm->read (inm);
  Tm->read (intr);
  
  //int. points allocation and initiation
  Tm->intpointalloc ();
  Tm->intpointinit ();

  //  allocation of auxiliary arrays on integration points
  Mm->init_ip_1 ();

  if (Cp->tprob==fully_coupled_mech_trans){
    //Cmu->read (in->file);
    //Cml->read (in->file);
    Cmu->read (in);
    Cml->read (in);
  }

  //  input of cross section characteristics
  Mc->read (inm);
  Tc->read (intr);
  
  //  input of loading
  Mb->read (inm);
  Tb->read (intr);

  //read mefel function
  if (Mp->tprob == growing_mech_structure){
    Gtm->read_gf (inm);
    Mt->alloc_growstr ();
  }
 
  //  prescribed values on elements investigation
  //Tt->elemprescvalue();
  //  searching for elements with nodes where source is defined
  Tb->elemsource ();

  if (Cp->tprob==fully_coupled_mech_trans){  
    //  left and right hand sides
    Lsrsc = new lhsrhsc;
    Lsrsc->alloc ();
    //Lsrs->initcond (in);//proc??!!
    
    Lsrs = new lhsrhs;
    Lsrst = new lhsrhst;
    
    Lsrst->lhsi=Lsrsc->lhsi;
    Lsrst->initcond (intr);
  }

  if ((Cp->tprob==par_coupl_mech_trans) || (Cp->tprob==growing_par_coupl_mech_trans)){
    //  object for left and right hand side of MEFEL problem
    Lsrs = new lhsrhs;
    Lsrs->alloc ();
    Lsrs->initcond (in);
    
    //  object for left and right hand side of TRFEL problem
    Lsrst = new lhsrhst;
    Lsrst->alloc ();
    Lsrst->initcond (intr);
    
    
    Mb->alloc_sumcomp ();
    
    //  allocation of array for temperature
    Mm->tempr = new double [Mm->tnip];
    //  allocation of array for initial temperature
    Mm->inittempr = new double [Mm->tnip];
    //  allocation of array for moisture
    Mm->moist = new double [Mm->tnip];

    for (i=0;i<Mm->tnip;i++){
      Mm->tempr[i]=0.0;
      Mm->inittempr[i]=0.0;
      Mm->moist[i]=0.0;
    }
  }
  
  // input of output description
  //metr
  Outdc->read(in);
  //mefel
  Outdm->read(inm);
  //trfel
  Outdt->read(intr);

  fclose (Outc);
  fclose (Outt);
  fclose (Out);
  
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  Cp->filename_decomposition (Outdc->outfn,path,name,suffix);
  sprintf(fname, "%s%s.log", path, name);
  Outc = fopen(fname,"wt");
  delete [] path; delete [] name; delete [] suffix;
  Tp->filename_decompositiont (Outdt->outfn,path,name,suffix);
  sprintf(fname, "%s%s.log", path, name);
  Outt = fopen(fname,"wt");
  delete [] path; delete [] name; delete [] suffix;
  Mp->filename_decomposition (Cp->minfile,path,name,suffix);
  if ((strlen(path) == 0) && (strlen(name) == 0))
  {
    sprintf(fname, "mefel.log");
    Out = fopen(fname,"at");
  }
  else
  {
    sprintf(fname, "%s%s.log", path, name);
    Out = fopen(fname,"wt");
  }
  fprintf(stdout, "\n Log file name : '%s'", fname);
  delete [] path; delete [] name; delete [] suffix;
  
  //Mm->stra.init(strain);
  //Mm->stre.init(stress);

  //  allocation of auxiliary arrays on integration points
  Mm->init_ip_2 ();

  //  nodes allocation
  Tt->alloc_nodes ();
  Mt->alloc_nodes_arrays ();

  Mt->give_maxncompstr(Mm->max_ncompstrn, Mm->max_ncompstre);
  Mt->give_maxncompo(Mm->max_nncompo, Mm->max_encompo);

  //if (Mp->straincomp==1){
  //Mm->stra.alloc(Mb->nlc);
  //}
  //if (Mp->stresscomp==1){
  //Mm->stre.alloc(Mb->nlc);
  //}

  //  prescribed displacements on elements investigation
  Mt->elemprescdisp ();
  //  reactions on elements investigation
  if (Mp->reactcomp==1){
    Mt->comreac ();
  }

  //  computation of initial values at integration points from initial nodal values
  if (Mb->ico > 0)
    Mb->inicipval();
    
  if (Mp->matmodel == nonlocal){    
    //  list of adjacent elements
    Gtm->adjacelem (Out);
    //  list of adjacent integration points was not restored 
    //  from file it is necessary to create it from scratch
    if (restore_adjacip() == 0)
      adjacip ();
    save_adjacip();
    ipvolume ();
  }

  //  allocation of eigenstrains
  //  eigenstrains contain strains generated by coupling terms
  Mm->alloceigstrains ();
  Mm->alloctempstrains ();
  Mm->alloceigstresses ();

  //Mt->nodedisplacements (); //tohle jeste rozmyslet??!!





  xfclose (in);
  xfclose (inm);  
  xfclose (intr);
}
