#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "probdescc.h"
#include "globalc.h"
#include "gmatrix.h"
#include "intools.h"



probdescc::probdescc (void)
{
  niilnr=0;  errnr=0.0;
  alpha=0.0;  time=0.0;
  zero = 1.0e-20;

  
  lbb = notdeflbb;

  //  smaller number of integration points
  savemode=1;
  
  //  type of nonlinear solver
  fcsolv = (coupsolver) 0;
  
  //  type of residuum
  restype = (residuumtype) 0;
  
  //  solver of system of linear algebraic equations
  ssle = new slesolv();
}

probdescc::~probdescc (void)
{
  delete [] path; delete [] filename; delete [] name; delete [] suffix;
}

/**
   function reads basic data about solved problem
   
   @param in - input stream
   
   23.11.2002
*/
void probdescc::read (XFILE *in)
{
  //  problem name
  //inputln (in->file,name,1000);
  xfscanf (in,"%a",name);

  //  MEFEL input file name
  //inputln (in->file,minfile,1000);
  xfscanf (in,"%a",minfile);

  //  TRFEL input file name
  //inputln (in->file,tinfile,1000);
  xfscanf (in,"%a",tinfile);

  //  detail messages
  Mesprc=0;
  xfscanf (in,"%k%ld","mesprc",&Mesprc);
  if (Mesprc==1)  fprintf (stdout,"\n detail information will be printed\n");
  else fprintf (stdout,"\n only important messages will be printed\n");
 
  //  problem type
  xfscanf (in,"%k%m","problemtype",&problemtypec_kwdset,(int*)&tprob);
  //  Ladyzhenskaja-Babuska-Brezzi
  xfscanf (in,"%k%m","LBB",&ladbabuskabrezzi_kwdset,(int*)&lbb);
  //  transported matter
  xfscanf (in,"%k%m","transmatter",&transmattert_kwdset,(int*)&tmatt);
  //  names of transported media
  xfscanf (in,"%k%m","mednames",&mednamest_kwdset,(int*)&mednam);
  
  switch (tprob){
  case fully_coupled_mech_trans:{
    
    //  time controller
    //  start time, end time, number of important times
    //  list of important times, time increment function
    timecon.read (in);
    
    //  alpha coefficient
    xfscanf (in,"%lf",&alpha);
    
    //  parameters of Newton-Raphson method
    //  maximum number of iteration in one increment
    //  maximum norm of residual vector
    xfscanf (in,"%d",(int*)&tnlinsol);
    xfscanf (in,"%ld %lf",&niilnr,&errnr);
    
    //  type of nonlinear solver
    xfscanf (in,"%d %d",(int*)&fcsolv,(int*)&restype);
        
    //  type of storage of zero-order and first order matrices
    //  zero-order matrix = stiffness and conductivity matrices
    //  first-order matrix = capacity matrix
    xfscanf (in,"%k%m%k%m","zeroordermat",&storagetype_kwdset,(int*)&tstord0,"firstordermat",&storagetype_kwdset,(int*)&tstord1);
    
    //  data about solver of system of linear equations
    ssle->read (in,Mesprc);
    
    break;
  }
  case growing_par_coupl_mech_trans:
  case par_coupl_mech_trans:{
    
    //  time controller
    //  start time, end time, number of important times
    //  list of important times, time increment function
    timecon.read (in);
    
    xfscanf (in,"%k%m","nonlinsolvertype",&nonlinsolvertypec_kwdset,(int*)&tnlinsol);
    xfscanf (in,"%ld %lf",&niilnr,&errnr);
    
    //  type of coupled solver
    xfscanf (in,"%k%m","coupledsolver",&coupsolver_kwdset,(int*)&fcsolv);
    
    //  this problem requires storage of actual nodal values on nodes
    Tp->nvs=1;
    //  this problem requires storage of actual nodal values on nodes
    Tp->pnvs=1;
    //  this problem requires storage of initial nodal values on nodes
    Tp->invs=1;
    //  this problem requires storage of time derivatives of nodal values on nodes
    //tdnvs=1;

    break;
  }
  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
    abort ();
  }
  }
}


/**  function prints basic data about solved problem
     JK, 25.9.2001
     
     @param in - output file
*/
void probdescc::print (FILE *out)
{
  //  problem name
  fprintf (out, "%s\n", name);

  //  MEFEL input file name
  fprintf (out,"%a\n",minfile);

  //  TRFEL input file name
  fprintf (out,"%a\n",tinfile);

  //  detail messages
  fprintf (out,"%ld\n", Mesprc);

  //  problem type
  fprintf (out,"%d", tprob);
  
  // LBB condition
  fprintf (out,"%d", lbb);

  //  transported matter
  fprintf (out," %d", tmatt);
  
  //  names of transported media
  fprintf (out, " %d\n", mednam);
     
  switch (tprob){
  case fully_coupled_mech_trans:{
    
    timecon.print (out);

    fprintf (out," %d",tnlinsol);

    fprintf (out, " %lf \n", alpha);

    fprintf (out," %ld %lf\n",niilnr,errnr);
    
    fprintf (out," %d %d\n",fcsolv,restype);

    fprintf (out," %d %d\n",tstord0,tstord1);
    
    ssle->print (out);
    
    fprintf (out,"\n");
    
    break;
  }

  case growing_par_coupl_mech_trans:
  case par_coupl_mech_trans:{
    
    timecon.print (out);
    
    
    fprintf (out," %d",tnlinsol);

    fprintf (out," %ld %lf\n",niilnr,errnr);
    
    fprintf (out," %d\n",fcsolv);

    break;
  }

  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }  
}

/**
   Function decomposes name of file(with path) to path,filemane and suffix.
   ppp/ppp/fff.sss.sss -> path = "ppp/ppp" , name = "fff" , suffix = ".sss.sss"
   
   @param file - decomposed name of file
   @param path,name,suffix - components of name of file
   
   created  1.4.2003, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void probdescc::filename_decomposition (const char *file,char *&pathc,char *&namec,char *&suffixc)
{
  char *fix;
  pathc = new char[strlen(file)+1];
  namec = new char[strlen(file)+1];
  suffixc = new char[strlen(file)+1];
  
  strcpy (pathc,file);
  fix = strrchr(pathc,'/');
  if (fix==NULL){
    strcpy (namec,pathc);
    pathc[0] = '\0';
  }
  else{
    strcpy (namec,fix+1);
    fix[1] = '\0';
  }
  
  fix = strchr(namec,'.');
  if (fix==NULL)
    suffixc[0] = '\0';
  else{
    strcpy (suffixc,fix);
    fix[0] = '\0';
  }
}
